@@ -58,8 +58,8 @@ macro_rules! impl_dyn_series {
                 self.0.cummin(reverse).$into_logical().into_series()
             }
 
-            fn _set_sorted(&mut self, is_sorted: IsSorted) {
-                self.0.deref_mut().set_sorted2(is_sorted)
+            fn _set_sorted_flag(&mut self, is_sorted: IsSorted) {
+                self.0.deref_mut().set_sorted_flag(is_sorted)
             }
 
             #[cfg(feature = "zip_with")]
@@ -124,10 +124,11 @@ macro_rules! impl_dyn_series {
                         let rhs = rhs.cast(&dt)?;
                         lhs.subtract(&rhs)
                     }
-                    (DataType::Date, DataType::Duration(tu)) => {
-                        ((&self.cast(&DataType::Datetime(*tu, None)).unwrap()) - rhs)
-                            .cast(&DataType::Date)
-                    }
+                    (DataType::Date, DataType::Duration(_)) => ((&self
+                        .cast(&DataType::Datetime(TimeUnit::Milliseconds, None))
+                        .unwrap())
+                        - rhs)
+                        .cast(&DataType::Date),
                     (dtl, dtr) => Err(PolarsError::ComputeError(
                         format!(
                             "cannot do subtraction on these date types: {:?}, {:?}",
@@ -139,10 +140,11 @@ macro_rules! impl_dyn_series {
             }
             fn add_to(&self, rhs: &Series) -> PolarsResult<Series> {
                 match (self.dtype(), rhs.dtype()) {
-                    (DataType::Date, DataType::Duration(tu)) => {
-                        ((&self.cast(&DataType::Datetime(*tu, None)).unwrap()) + rhs)
-                            .cast(&DataType::Date)
-                    }
+                    (DataType::Date, DataType::Duration(_)) => ((&self
+                        .cast(&DataType::Datetime(TimeUnit::Milliseconds, None))
+                        .unwrap())
+                        + rhs)
+                        .cast(&DataType::Date),
                     (dtl, dtr) => Err(PolarsError::ComputeError(
                         format!(
                             "cannot do addition on these date types: {:?}, {:?}",
@@ -177,10 +179,10 @@ macro_rules! impl_dyn_series {
         }
 
         impl SeriesTrait for SeriesWrap<$ca> {
-            fn is_sorted(&self) -> IsSorted {
-                if self.0.is_sorted() {
+            fn is_sorted_flag(&self) -> IsSorted {
+                if self.0.is_sorted_flag() {
                     IsSorted::Ascending
-                } else if self.0.is_sorted_reverse() {
+                } else if self.0.is_sorted_reverse_flag() {
                     IsSorted::Descending
                 } else {
                     IsSorted::Not
@@ -290,8 +292,9 @@ macro_rules! impl_dyn_series {
             unsafe fn take_unchecked(&self, idx: &IdxCa) -> PolarsResult<Series> {
                 let mut out = ChunkTake::take_unchecked(self.0.deref(), idx.into());
 
-                if self.0.is_sorted() && (idx.is_sorted() || idx.is_sorted_reverse()) {
-                    out.set_sorted2(idx.is_sorted2())
+                if self.0.is_sorted_flag() && (idx.is_sorted_flag() || idx.is_sorted_reverse_flag())
+                {
+                    out.set_sorted_flag(idx.is_sorted_flag2())
                 }
 
                 Ok(out.$into_logical().into_series())
@@ -338,14 +341,14 @@ macro_rules! impl_dyn_series {
                 }
             }
 
-            fn get(&self, index: usize) -> AnyValue {
+            fn get(&self, index: usize) -> PolarsResult<AnyValue> {
                 self.0.get_any_value(index)
             }
 
             #[inline]
             #[cfg(feature = "private")]
             unsafe fn get_unchecked(&self, index: usize) -> AnyValue {
-                self.0.get_any_value_unchecked(index).$into_logical()
+                self.0.get_any_value_unchecked(index)
             }
 
             fn sort_with(&self, options: SortOptions) -> Series {
