@@ -1,3 +1,5 @@
+#[cfg(feature = "dtype-binary")]
+mod binary;
 #[cfg(feature = "interpolate")]
 mod interpolate;
 mod list;
@@ -10,6 +12,8 @@ mod to_dummies;
 #[cfg(feature = "top_k")]
 mod top_k;
 
+#[cfg(feature = "dtype-binary")]
+pub use binary::*;
 #[cfg(feature = "interpolate")]
 pub use interpolate::*;
 pub use list::*;
