@@ -18,7 +18,7 @@ pub(super) fn sign(s: &Series) -> PolarsResult<Series> {
             sign(&s)
         }
         dt => Err(PolarsError::ComputeError(
-            format!("cannot use 'sign' on Series of dtype: {:?}", dt).into(),
+            format!("cannot use 'sign' on Series of dtype: {dt:?}").into(),
         )),
     }
 }
