@@ -11,20 +11,24 @@ from typing import (
     Generator,
     Iterable,
     Mapping,
+    MutableMapping,
     Sequence,
     get_type_hints,
 )
 
 from polars import internals as pli
 from polars.datatypes import (
+    N_INFER_DEFAULT,
     Categorical,
-    ColumnsType,
     Date,
     Datetime,
     Duration,
     Float32,
     List,
     PolarsDataType,
+    SchemaDefinition,
+    SchemaDict,
+    Struct,
     Time,
     Unknown,
     Utf8,
@@ -50,7 +54,7 @@ from polars.dependencies import numpy as np
 from polars.dependencies import pandas as pd
 from polars.dependencies import pyarrow as pa
 from polars.exceptions import ShapeError
-from polars.utils import _is_generator, arrlen, threadpool_size
+from polars.utils import _is_generator, arrlen, range_to_series, threadpool_size
 
 if version_info >= (3, 10):
 
@@ -83,8 +87,8 @@ def is_namedtuple(value: Any, annotated: bool = False) -> bool:
 
 
 def include_unknowns(
-    schema: dict[str, PolarsDataType], cols: Sequence[str]
-) -> dict[str, PolarsDataType]:
+    schema: SchemaDict, cols: Sequence[str]
+) -> MutableMapping[str, PolarsDataType]:
     """Complete partial schema dict by including Unknown type."""
     return {col: schema.get(col, Unknown) for col in cols}
 
@@ -269,6 +273,8 @@ def sequence_to_pyseries(
     if value is not None:
         if is_dataclass(value) or is_namedtuple(value, annotated=True):
             return pli.DataFrame(values).to_struct(name)._s
+        elif isinstance(value, range):
+            values = [range_to_series("", v) for v in values]
         else:
             # for temporal dtypes:
             # * if the values are integer, we take the physical branch.
@@ -283,13 +289,27 @@ def sequence_to_pyseries(
 
     # physical branch
     # flat data
-    if (dtype is not None) and is_polars_dtype(dtype) and (python_dtype is None):
+    if (
+        dtype is not None
+        and dtype not in (Struct, Unknown)
+        and is_polars_dtype(dtype)
+        and (python_dtype is None)
+    ):
         constructor = polars_type_to_constructor(dtype)
         pyseries = constructor(name, values, strict)
 
         if dtype in (Date, Datetime, Duration, Time, Categorical):
             pyseries = pyseries.cast(dtype, True)
         return pyseries
+
+    elif dtype == Struct:
+        struct_schema = dtype.to_schema() if isinstance(dtype, Struct) else None
+        empty = {}  # type: ignore[var-annotated]
+        return sequence_to_pydf(
+            data=[(empty if v is None else v) for v in values],
+            columns=struct_schema,
+            orient="row",
+        ).to_struct(name)
     else:
         if python_dtype is None:
             if value is None:
@@ -319,7 +339,12 @@ def sequence_to_pyseries(
         elif python_dtype in (list, tuple):
             if nested_dtype is None:
                 nested_value = _get_first_non_none(value)
-                nested_dtype = type(nested_value) if nested_value is not None else float
+                if isinstance(nested_value, range):
+                    nested_dtype = list
+                else:
+                    nested_dtype = (
+                        type(nested_value) if nested_value is not None else float
+                    )
 
             # recursively call Series constructor
             if nested_dtype == list:
@@ -333,11 +358,10 @@ def sequence_to_pyseries(
                     strict=strict,
                 )
 
-            # logs will show a panic if we infer wrong dtype
-            # and its hard to error from rust side
-            # to reduce the likelihood of this happening
-            # we infer the dtype of first 100 elements
-            # if all() fails, we will hit the PySeries.new_object
+            # logs will show a panic if we infer wrong dtype and it's hard to error
+            # from the rust side. to reduce the likelihood of this happening we
+            # infer the dtype of first 100 elements; if all() fails, we will hit
+            # the PySeries.new_object
             if not _PYARROW_AVAILABLE:
                 # check lists for consistent inner types
                 if isinstance(value, list):
@@ -391,7 +415,6 @@ def sequence_to_pyseries(
             return PySeries.new_series_list(name, values, strict)
         else:
             constructor = py_type_to_constructor(python_dtype)
-
             if constructor == PySeries.new_object:
                 try:
                     return PySeries.new_from_anyvalues(name, values)
@@ -453,7 +476,7 @@ def _pandas_series_to_arrow(
 
     """
     dtype = getattr(values, "dtype", None)
-    if dtype == "object" and len(values) > 0:
+    if dtype == "object":
         first_non_none = _get_first_non_none(values.values)  # type: ignore[arg-type]
 
         if isinstance(first_non_none, str):
@@ -491,7 +514,7 @@ def pandas_to_pyseries(
 
 
 def _handle_columns_arg(
-    data: list[PySeries], columns: Sequence[str] | None = None
+    data: list[PySeries], columns: Sequence[str] | None = None, from_dict: bool = False
 ) -> list[PySeries]:
     """Rename data according to columns argument."""
     if not columns:
@@ -500,6 +523,11 @@ def _handle_columns_arg(
         if not data:
             return [pli.Series(c, None)._s for c in columns]
         elif len(data) == len(columns):
+            if from_dict:
+                series_map = {s.name(): s for s in data}
+                if all((col in series_map) for col in columns):
+                    return [series_map[col] for col in columns]
+
             for i, c in enumerate(columns):
                 data[i].rename(c)
             return data
@@ -508,35 +536,54 @@ def _handle_columns_arg(
 
 
 def _post_apply_columns(
-    pydf: PyDataFrame, columns: ColumnsType, categoricals: set[str] | None = None
+    pydf: PyDataFrame,
+    columns: SchemaDefinition | None,
+    structs: dict[str, Struct] | None = None,
+    schema_overrides: SchemaDict | None = None,
 ) -> PyDataFrame:
     """Apply 'columns' param _after_ PyDataFrame creation (if no alternative)."""
     pydf_columns, pydf_dtypes = pydf.columns(), pydf.dtypes()
-    columns, dtypes = _unpack_columns(columns or pydf_columns)
+    columns, dtypes = _unpack_columns(
+        (columns or pydf_columns), schema_overrides=schema_overrides
+    )
+    column_subset: list[str] = []
     if columns != pydf_columns:
-        pydf.set_column_names(columns)
+        if len(columns) < len(pydf_columns) and columns == pydf_columns[: len(columns)]:
+            column_subset = columns
+        else:
+            pydf.set_column_names(columns)
 
     column_casts = []
     for i, col in enumerate(columns):
-        if categoricals and col in categoricals:
+        if dtypes.get(col) == Categorical != pydf_dtypes[i]:
             column_casts.append(pli.col(col).cast(Categorical)._pyexpr)
+        elif structs and col in structs and structs[col] != pydf_dtypes[i]:
+            column_casts.append(pli.col(col).cast(structs[col])._pyexpr)
         elif col in dtypes and dtypes[col] != pydf_dtypes[i]:
             column_casts.append(pli.col(col).cast(dtypes[col])._pyexpr)
 
-    if column_casts:
-        pydf = pydf.lazy().with_columns(column_casts).collect()
+    if column_casts or column_subset:
+        pydf = pydf.lazy()
+        if column_casts:
+            pydf = pydf.with_columns(column_casts)
+        if column_subset:
+            pydf = pydf.select([pli.col(col)._pyexpr for col in column_subset])
+        pydf = pydf.collect()
+
     return pydf
 
 
 def _unpack_columns(
-    columns: ColumnsType | None,
-    lookup_names: Iterable[str] | None = None,
+    columns: SchemaDefinition | None,
+    schema_overrides: SchemaDict | None = None,
     n_expected: int | None = None,
-) -> tuple[list[str], dict[str, PolarsDataType]]:
+    lookup_names: Iterable[str] | None = None,
+) -> tuple[list[str], SchemaDict]:
     """
     Unpack column names and create dtype lookup.
 
-    Works for any (name, dtype) pairs or schema dict input.
+    Works for any (name, dtype) pairs or schema dict input,
+    overriding any inferred dtypes with explicit dtypes if supplied.
     """
     if isinstance(columns, dict):
         columns = list(columns.items())
@@ -549,22 +596,77 @@ def _unpack_columns(
     lookup = {
         col: name for col, name in zip_longest(column_names, lookup_names or []) if name
     }
+    column_dtypes = {
+        lookup.get(col[0], col[0]): col[1]
+        for col in (columns or [])
+        if not isinstance(col, str) and col[1]
+    }
+    if schema_overrides:
+        column_dtypes.update(schema_overrides)
+
     return (
         column_names or None,  # type: ignore[return-value]
-        {
-            lookup.get(col[0], col[0]): col[1]  # type: ignore[misc]
-            for col in (columns or [])
-            if not isinstance(col, str) and col[1]
-        },
+        column_dtypes,
     )
 
 
+def _expand_dict_scalars(
+    data: Mapping[str, Sequence[object] | Mapping[str, Sequence[object]] | pli.Series],
+    schema_overrides: SchemaDict | None = None,
+    order: Sequence[str] | None = None,
+) -> dict[str, pli.Series]:
+    """Expand scalar values in dict data (propagate literal as array)."""
+    updated_data = {}
+    if data:
+        dtypes = schema_overrides or {}
+        array_len = max((arrlen(val) or 0) for val in data.values())
+        if array_len > 0:
+            for name, val in data.items():
+                dtype = dtypes.get(name)
+                if isinstance(val, dict) and dtype != Struct:
+                    updated_data[name] = pli.DataFrame(val).to_struct(name)
+
+                elif arrlen(val) is not None or _is_generator(val):
+                    updated_data[name] = pli.Series(name=name, values=val, dtype=dtype)
+
+                elif val is None or isinstance(  # type: ignore[redundant-expr]
+                    val, (int, float, str, bool)
+                ):
+                    updated_data[name] = pli.Series(
+                        name=name, values=[val], dtype=dtype
+                    ).extend_constant(val, array_len - 1)
+                else:
+                    updated_data[name] = pli.Series(
+                        name=name, values=[val] * array_len, dtype=dtype
+                    )
+
+        elif all((arrlen(val) == 0) for val in data.values()):
+            for name, val in data.items():
+                updated_data[name] = pli.Series(
+                    name, values=val, dtype=dtypes.get(name)
+                )
+
+        elif all((arrlen(val) is None) for val in data.values()):
+            for name, val in data.items():
+                updated_data[name] = pli.Series(
+                    name,
+                    values=(val if _is_generator(val) else [val]),
+                    dtype=dtypes.get(name),
+                )
+    if order and list(updated_data) != order:
+        return {col: updated_data.pop(col) for col in order}
+    return updated_data
+
+
 def dict_to_pydf(
-    data: Mapping[str, Sequence[object] | Mapping[str, Sequence[object]]],
-    columns: ColumnsType | None = None,
+    data: Mapping[str, Sequence[object] | Mapping[str, Sequence[object]] | pli.Series],
+    columns: SchemaDefinition | None = None,
+    schema_overrides: SchemaDict | None = None,
 ) -> PyDataFrame:
     """Construct a PyDataFrame from a dictionary of sequences."""
-    if columns is not None:
+    if not columns:
+        columns = list(data)
+    if columns:
         # the columns arg may also set the dtype/column order of the series
         if isinstance(columns, dict) and data:
             if not all((col in columns) for col in data):
@@ -573,18 +675,19 @@ def dict_to_pydf(
                 )
             data = {col: data[col] for col in columns}
 
-        columns, dtypes = _unpack_columns(columns, lookup_names=data.keys())
-
-        if not data and dtypes:
+        columns, schema_overrides = _unpack_columns(
+            columns, lookup_names=data.keys(), schema_overrides=schema_overrides
+        )
+        if not data and schema_overrides:
             data_series = [
-                pli.Series(name, [], dtypes.get(name))._s for name in columns
+                pli.Series(name, [], schema_overrides.get(name))._s for name in columns
             ]
         else:
             data_series = [
-                pli.Series(name, values, dtypes.get(name))._s
-                for name, values in data.items()
+                s._s for s in _expand_dict_scalars(data, schema_overrides).values()
             ]
-        data_series = _handle_columns_arg(data_series, columns=columns)
+
+        data_series = _handle_columns_arg(data_series, columns=columns, from_dict=True)
         return PyDataFrame(data_series)
 
     if _NUMPY_AVAILABLE:
@@ -615,52 +718,51 @@ def dict_to_pydf(
                 )
             return PyDataFrame(data_series)
 
-    if data:
-        # expand scalars (convenience init for propagating literals)
-        array_len = max((arrlen(val) or 0) for val in data.values())
-        if array_len > 0:
-            for key, val in data.items():
-                if arrlen(val) is None and not _is_generator(val):
-                    data[key] = [val] * array_len  # type: ignore[index]
-        elif all((arrlen(val) is None) for val in data.values()):
-            for key, val in data.items():
-                if not _is_generator(val):
-                    data[key] = [val]  # type: ignore[index]
-
-    # fast path
+    data = _expand_dict_scalars(data)
     return PyDataFrame.read_dict(data)
 
 
 def sequence_to_pydf(
     data: Sequence[Any],
-    columns: ColumnsType | None = None,
+    columns: SchemaDefinition | None = None,
+    schema_overrides: SchemaDict | None = None,
     orient: Orientation | None = None,
-    infer_schema_length: int | None = 50,
+    infer_schema_length: int | None = N_INFER_DEFAULT,
 ) -> PyDataFrame:
     """Construct a PyDataFrame from a sequence."""
     data_series: list[PySeries]
-
     if len(data) == 0:
-        return dict_to_pydf({}, columns=columns)
+        return dict_to_pydf({}, columns=columns, schema_overrides=schema_overrides)
+
     if isinstance(data[0], Generator):
         data = [list(row) for row in data]
 
     if isinstance(data[0], pli.Series):
         series_names = [s.name for s in data]
-        columns, dtypes = _unpack_columns(columns or series_names, n_expected=len(data))
+        columns, schema_overrides = _unpack_columns(
+            columns or series_names,
+            schema_overrides=schema_overrides,
+            n_expected=len(data),
+        )
         data_series = []
         for i, s in enumerate(data):
             if not s.name:  # TODO: Replace by `if s.name is None` once allowed
                 s.rename(columns[i], in_place=True)
-            new_dtype = dtypes.get(columns[i])
+            new_dtype = schema_overrides.get(columns[i])
             if new_dtype and new_dtype != s.dtype:
                 s = s.cast(new_dtype)
             data_series.append(s._s)
 
     elif isinstance(data[0], dict):
-        column_names, dtypes = _unpack_columns(columns)
-        schema_overrides = include_unknowns(dtypes, column_names) if dtypes else None
-        pydf = PyDataFrame.read_dicts(data, infer_schema_length, schema_overrides)
+        column_names, schema_overrides = _unpack_columns(
+            columns, schema_overrides=schema_overrides
+        )
+        dtypes = (
+            include_unknowns(schema_overrides, column_names)
+            if schema_overrides
+            else None
+        )
+        pydf = PyDataFrame.read_dicts(data, infer_schema_length, dtypes)
         if column_names:
             pydf = _post_apply_columns(pydf, column_names)
         return pydf
@@ -681,29 +783,38 @@ def sequence_to_pydf(
             orient = "col" if len(columns) == len(data) else "row"
 
         if orient == "row":
-            column_names, dtypes = _unpack_columns(columns)
-            schema_override = include_unknowns(dtypes, column_names) if dtypes else {}
+            column_names, schema_overrides = _unpack_columns(
+                columns, schema_overrides=schema_overrides, n_expected=len(data[0])
+            )
+            schema_override = (
+                include_unknowns(schema_overrides, column_names)
+                if schema_overrides
+                else {}
+            )
             if column_names and data and len(data[0]) != len(column_names):
                 raise ShapeError("The row data does not match the number of columns")
-            categoricals = {
-                col for col, tp in schema_override.items() if tp == Categorical
-            }
-            for col in categoricals:
-                schema_override[col] = Utf8
+
+            for col, tp in schema_override.items():
+                if tp == Categorical:
+                    schema_override[col] = Utf8
 
             pydf = PyDataFrame.read_rows(
                 data,
                 infer_schema_length,
                 schema_override or None,
             )
-            if column_names:
-                pydf = _post_apply_columns(pydf, column_names, categoricals)
+            if column_names or schema_overrides:
+                pydf = _post_apply_columns(
+                    pydf, column_names, schema_overrides=schema_overrides
+                )
             return pydf
 
         elif orient == "col" or orient is None:
-            columns, dtypes = _unpack_columns(columns, n_expected=len(data))
+            columns, schema_overrides = _unpack_columns(
+                columns, schema_overrides=schema_overrides, n_expected=len(data)
+            )
             data_series = [
-                pli.Series(columns[i], data[i], dtypes.get(columns[i]))._s
+                pli.Series(columns[i], data[i], schema_overrides.get(columns[i]))._s
                 for i in range(len(data))
             ]
         else:
@@ -713,36 +824,49 @@ def sequence_to_pydf(
 
     elif is_dataclass(data[0]):
         if columns:
-            columns, dtypes = _unpack_columns(columns)
-            schema_override = {col: dtypes.get(col, Unknown) for col in columns}
+            columns, schema_overrides = _unpack_columns(
+                columns, schema_overrides=schema_overrides
+            )
+            schema_override = {
+                col: schema_overrides.get(col, Unknown) for col in columns
+            }
         else:
             columns = None
             schema_override = {
                 col: (py_type_to_dtype(tp, raise_unmatched=False) or Unknown)
                 for col, tp in dataclass_type_hints(data[0].__class__).items()
             }
-        categoricals = {col for col, tp in schema_override.items() if tp == Categorical}
-        for col in categoricals:
-            schema_override[col] = Utf8
+            schema_override.update(schema_overrides or {})
+
+        for col, tp in schema_override.items():
+            if tp == Categorical:
+                schema_override[col] = Utf8
 
         pydf = PyDataFrame.read_rows(
             [astuple(dc) for dc in data], infer_schema_length, schema_override or None
         )
-        if columns:
-            pydf = _post_apply_columns(pydf, columns, categoricals)
+        if schema_override:
+            structs = {
+                col: tp for col, tp in schema_override.items() if isinstance(tp, Struct)
+            }
+            pydf = _post_apply_columns(
+                pydf, columns, structs, schema_overrides=schema_overrides
+            )
         return pydf
 
     elif _check_for_pandas(data[0]) and isinstance(
         data[0], (pd.Series, pd.DatetimeIndex)
     ):
-        dtypes = {}
         if columns is not None:
-            columns, dtypes = _unpack_columns(columns, n_expected=1)
+            columns, schema_overrides = _unpack_columns(
+                columns, schema_overrides=schema_overrides, n_expected=1
+            )
 
+        schema_overrides = schema_overrides or {}
         data_series = []
         for i, s in enumerate(data):
             name = columns[i] if columns else s.name
-            dtype = dtypes.get(name, None)
+            dtype = schema_overrides.get(name, None)
             pyseries = pandas_to_pyseries(name=name, values=s)
             if dtype is not None and dtype != pyseries.dtype():
                 pyseries = pyseries.cast(dtype, strict=True)
@@ -750,8 +874,12 @@ def sequence_to_pydf(
 
         columns = None
     else:
-        columns, dtypes = _unpack_columns(columns, n_expected=1)
-        data_series = [pli.Series(columns[0], data, dtypes.get(columns[0]))._s]
+        columns, schema_overrides = _unpack_columns(
+            columns, schema_overrides=schema_overrides, n_expected=1
+        )
+        data_series = [
+            pli.Series(columns[0], data, schema_overrides.get(columns[0]))._s
+        ]
 
     data_series = _handle_columns_arg(data_series, columns=columns)
     return PyDataFrame(data_series)
@@ -759,7 +887,8 @@ def sequence_to_pydf(
 
 def numpy_to_pydf(
     data: np.ndarray[Any, Any],
-    columns: ColumnsType | None = None,
+    columns: SchemaDefinition | None = None,
+    schema_overrides: SchemaDict | None = None,
     orient: Orientation | None = None,
 ) -> PyDataFrame:
     """Construct a PyDataFrame from a numpy ndarray."""
@@ -796,7 +925,6 @@ def numpy_to_pydf(
             raise ValueError(
                 f"orient must be one of {{'col', 'row', None}}, got {orient} instead."
             )
-
     else:
         raise ValueError(
             "Cannot create DataFrame from numpy array with more than two dimensions."
@@ -805,24 +933,28 @@ def numpy_to_pydf(
     if columns is not None and len(columns) != n_columns:
         raise ValueError("Dimensions of columns arg must match data dimensions.")
 
-    columns, dtypes = _unpack_columns(columns, n_expected=n_columns)
+    columns, schema_overrides = _unpack_columns(
+        columns, schema_overrides=schema_overrides, n_expected=n_columns
+    )
 
     # Convert data to series
     if shape == (0,):
         data_series = []
 
     elif len(shape) == 1:
-        data_series = [pli.Series(columns[0], data, dtypes.get(columns[0]))._s]
+        data_series = [
+            pli.Series(columns[0], data, schema_overrides.get(columns[0]))._s
+        ]
 
     else:
         if orient == "row":
             data_series = [
-                pli.Series(columns[i], data[:, i], dtypes.get(columns[i]))._s
+                pli.Series(columns[i], data[:, i], schema_overrides.get(columns[i]))._s
                 for i in range(n_columns)
             ]
         else:
             data_series = [
-                pli.Series(columns[i], data[i], dtypes.get(columns[i]))._s
+                pli.Series(columns[i], data[i], schema_overrides.get(columns[i]))._s
                 for i in range(n_columns)
             ]
 
@@ -831,18 +963,21 @@ def numpy_to_pydf(
 
 
 def arrow_to_pydf(
-    data: pa.Table, columns: ColumnsType | None = None, rechunk: bool = True
+    data: pa.Table,
+    columns: SchemaDefinition | None = None,
+    schema_overrides: SchemaDict | None = None,
+    rechunk: bool = True,
 ) -> PyDataFrame:
     """Construct a PyDataFrame from an Arrow Table."""
-    original_columns, dtypes = columns, None
-    if columns is not None:
-        columns, dtypes = _unpack_columns(columns)
-        try:
+    original_columns = columns
+    columns, schema_overrides = _unpack_columns(
+        (columns or data.column_names), schema_overrides=schema_overrides
+    )
+    try:
+        if columns and columns != data.column_names:
             data = data.rename_columns(columns)
-        except pa.lib.ArrowInvalid as e:
-            raise ValueError(
-                "Dimensions of columns arg must match data dimensions."
-            ) from e
+    except pa.lib.ArrowInvalid as e:
+        raise ValueError("Dimensions of columns arg must match data dimensions.") from e
 
     data_dict = {}
     # dictionaries cannot be built in different batches (categorical does not allow
@@ -904,18 +1039,35 @@ def arrow_to_pydf(
         df = df[names]
         pydf = df._df
 
-    if columns is not None and dtypes and original_columns:
-        pydf = _post_apply_columns(pydf, original_columns)
+    if columns != original_columns and (schema_overrides or original_columns):
+        pydf = _post_apply_columns(
+            pydf, original_columns, schema_overrides=schema_overrides
+        )
+    elif schema_overrides:
+        for col, dtype in zip(pydf.columns(), pydf.dtypes()):
+            override_dtype = schema_overrides.get(col)
+            if override_dtype is not None and dtype != override_dtype:
+                pydf = _post_apply_columns(
+                    pydf, original_columns, schema_overrides=schema_overrides
+                )
+                break
+
     return pydf
 
 
-def series_to_pydf(data: pli.Series, columns: ColumnsType | None = None) -> PyDataFrame:
+def series_to_pydf(
+    data: pli.Series,
+    columns: SchemaDefinition | None = None,
+    schema_overrides: SchemaDict | None = None,
+) -> PyDataFrame:
     """Construct a PyDataFrame from a Polars Series."""
     data_series = [data._s]
     series_name = [s.name() for s in data_series]
-    columns, dtypes = _unpack_columns(columns or series_name, n_expected=1)
-    if dtypes:
-        new_dtype = list(dtypes.values())[0]
+    columns, schema_overrides = _unpack_columns(
+        columns or series_name, schema_overrides=schema_overrides, n_expected=1
+    )
+    if schema_overrides:
+        new_dtype = list(schema_overrides.values())[0]
         if new_dtype != data.dtype:
             data_series[0] = data_series[0].cast(new_dtype, True)
 
@@ -925,24 +1077,32 @@ def series_to_pydf(data: pli.Series, columns: ColumnsType | None = None) -> PyDa
 
 def iterable_to_pydf(
     data: Iterable[Any],
-    columns: ColumnsType | None = None,
+    columns: SchemaDefinition | None = None,
+    schema_overrides: SchemaDict | None = None,
     orient: Orientation | None = None,
     chunk_size: int | None = None,
+    infer_schema_length: int | None = N_INFER_DEFAULT,
 ) -> PyDataFrame:
     """Construct a PyDataFrame from an iterable/generator."""
     original_columns = columns
-    dtypes: dict[str, PolarsDataType] = {}
     dtypes_by_idx: dict[int, PolarsDataType] = {}
     if columns is not None:
-        columns, dtypes = _unpack_columns(columns)
+        columns, schema_overrides = _unpack_columns(
+            columns, schema_overrides=schema_overrides
+        )
+    elif schema_overrides:
+        _columns, schema_overrides = _unpack_columns(
+            columns, schema_overrides=schema_overrides
+        )
 
     if not isinstance(data, Generator):
         data = iter(data)
 
     if orient == "col":
-        if columns is not None and dtypes:
+        if columns and schema_overrides:
             dtypes_by_idx = {
-                idx: dtypes.get(col, Unknown) for idx, col in enumerate(columns)
+                idx: schema_overrides.get(col, Unknown)
+                for idx, col in enumerate(columns)
             }
 
         return pli.DataFrame(
@@ -954,8 +1114,15 @@ def iterable_to_pydf(
             }
         )._df
 
-    def to_frame_chunk(values: list[Any], columns: ColumnsType | None) -> pli.DataFrame:
-        return pli.DataFrame(data=values, columns=columns, orient="row")
+    def to_frame_chunk(
+        values: list[Any], columns: SchemaDefinition | None
+    ) -> pli.DataFrame:
+        return pli.DataFrame(
+            data=values,
+            columns=columns,
+            orient="row",
+            infer_schema_length=infer_schema_length,
+        )
 
     n_chunks = 0
     n_chunk_elems = 1_000_000
@@ -991,7 +1158,8 @@ def iterable_to_pydf(
 
 def pandas_to_pydf(
     data: pd.DataFrame,
-    columns: ColumnsType | None = None,
+    columns: SchemaDefinition | None = None,
+    schema_overrides: SchemaDict | None = None,
     rechunk: bool = True,
     nan_to_none: bool = True,
 ) -> PyDataFrame:
@@ -1004,7 +1172,9 @@ def pandas_to_pydf(
         for col in data.columns
     }
     arrow_table = pa.table(arrow_dict)
-    return arrow_to_pydf(arrow_table, columns=columns, rechunk=rechunk)
+    return arrow_to_pydf(
+        arrow_table, columns=columns, schema_overrides=schema_overrides, rechunk=rechunk
+    )
 
 
 def coerce_arrow(array: pa.Array, rechunk: bool = True) -> pa.Array:
