@@ -1,5 +1,6 @@
 use polars::prelude::*;
 use polars_core::frame::row::{rows_to_schema_first_non_null, Row};
+use polars_core::series::SeriesIter;
 use pyo3::conversion::{FromPyObject, IntoPy};
 use pyo3::prelude::*;
 use pyo3::types::{PyBool, PyFloat, PyInt, PyList, PyString, PyTuple};
@@ -10,6 +11,17 @@ use crate::error::PyPolarsErr;
 use crate::series::PySeries;
 use crate::PyDataFrame;
 
+fn get_iters(df: &DataFrame) -> Vec<SeriesIter> {
+    df.get_columns().iter().map(|s| s.iter()).collect()
+}
+
+fn get_iters_skip(df: &DataFrame, skip: usize) -> Vec<std::iter::Skip<SeriesIter>> {
+    df.get_columns()
+        .iter()
+        .map(|s| s.iter().skip(skip))
+        .collect()
+}
+
 // the return type is Union[PySeries, PyDataFrame] and a boolean indicating if it is a dataframe or not
 pub fn apply_lambda_unknown<'a>(
     df: &'a DataFrame,
@@ -17,11 +29,11 @@ pub fn apply_lambda_unknown<'a>(
     lambda: &'a PyAny,
     inference_size: usize,
 ) -> PyResult<(PyObject, bool)> {
-    let columns = df.get_columns();
     let mut null_count = 0;
+    let mut iters = get_iters(df);
 
-    for idx in 0..df.height() {
-        let iter = columns.iter().map(|s: &Series| Wrap(s.get(idx)));
+    for _ in 0..df.height() {
+        let iter = iters.iter_mut().map(|it| Wrap(it.next().unwrap()));
         let arg = (PyTuple::new(py, iter),);
         let out = lambda.call1(arg)?;
 
@@ -134,13 +146,13 @@ fn apply_iter<'a, T>(
 where
     T: FromPyObject<'a>,
 {
-    let columns = df.get_columns();
-    ((init_null_count + skip)..df.height()).map(move |idx| {
-        let iter = columns.iter().map(|s: &Series| Wrap(s.get(idx)));
+    let mut iters = get_iters_skip(df, init_null_count + skip);
+    ((init_null_count + skip)..df.height()).map(move |_| {
+        let iter = iters.iter_mut().map(|it| Wrap(it.next().unwrap()));
         let tpl = (PyTuple::new(py, iter),);
         match lambda.call1(tpl) {
             Ok(val) => val.extract::<T>().ok(),
-            Err(e) => panic!("python function failed {}", e),
+            Err(e) => panic!("python function failed {e}"),
         }
     })
 }
@@ -157,7 +169,7 @@ where
     D: PyArrowPrimitiveType,
     D::Native: ToPyObject + FromPyObject<'a>,
 {
-    let skip = if first_value.is_some() { 1 } else { 0 };
+    let skip = usize::from(first_value.is_some());
     if init_null_count == df.height() {
         ChunkedArray::full_null("apply", df.height())
     } else {
@@ -174,7 +186,7 @@ pub fn apply_lambda_with_bool_out_type<'a>(
     init_null_count: usize,
     first_value: Option<bool>,
 ) -> ChunkedArray<BooleanType> {
-    let skip = if first_value.is_some() { 1 } else { 0 };
+    let skip = usize::from(first_value.is_some());
     if init_null_count == df.height() {
         ChunkedArray::full_null("apply", df.height())
     } else {
@@ -191,7 +203,7 @@ pub fn apply_lambda_with_utf8_out_type<'a>(
     init_null_count: usize,
     first_value: Option<&str>,
 ) -> Utf8Chunked {
-    let skip = if first_value.is_some() { 1 } else { 0 };
+    let skip = usize::from(first_value.is_some());
     if init_null_count == df.height() {
         ChunkedArray::full_null("apply", df.height())
     } else {
@@ -209,14 +221,13 @@ pub fn apply_lambda_with_list_out_type<'a>(
     first_value: Option<&Series>,
     dt: &DataType,
 ) -> PyResult<ListChunked> {
-    let columns = df.get_columns();
-
-    let skip = if first_value.is_some() { 1 } else { 0 };
+    let skip = usize::from(first_value.is_some());
     if init_null_count == df.height() {
         Ok(ChunkedArray::full_null("apply", df.height()))
     } else {
-        let iter = ((init_null_count + skip)..df.height()).map(|idx| {
-            let iter = columns.iter().map(|s: &Series| Wrap(s.get(idx)));
+        let mut iters = get_iters_skip(df, init_null_count + skip);
+        let iter = ((init_null_count + skip)..df.height()).map(|_| {
+            let iter = iters.iter_mut().map(|it| Wrap(it.next().unwrap()));
             let tpl = (PyTuple::new(py, iter),);
             match lambda.call1(tpl) {
                 Ok(val) => match val.getattr("_s") {
@@ -225,11 +236,11 @@ pub fn apply_lambda_with_list_out_type<'a>(
                         if val.is_none() {
                             None
                         } else {
-                            panic!("should return a Series, got a {:?}", val)
+                            panic!("should return a Series, got a {val:?}")
                         }
                     }
                 },
-                Err(e) => panic!("python function failed {}", e),
+                Err(e) => panic!("python function failed {e}"),
             }
         });
         iterator_to_list(dt, iter, init_null_count, first_value, "apply", df.height())
@@ -244,15 +255,15 @@ pub fn apply_lambda_with_rows_output<'a>(
     first_value: Row<'a>,
     inference_size: usize,
 ) -> PolarsResult<DataFrame> {
-    let columns = df.get_columns();
     let width = first_value.0.len();
     let null_row = Row::new(vec![AnyValue::Null; width]);
 
     let mut row_buf = Row::default();
 
     let skip = 1;
-    let mut row_iter = ((init_null_count + skip)..df.height()).map(|idx| {
-        let iter = columns.iter().map(|s: &Series| Wrap(s.get(idx)));
+    let mut iters = get_iters_skip(df, init_null_count + skip);
+    let mut row_iter = ((init_null_count + skip)..df.height()).map(|_| {
+        let iter = iters.iter_mut().map(|it| Wrap(it.next().unwrap()));
         let tpl = (PyTuple::new(py, iter),);
         match lambda.call1(tpl) {
             Ok(val) => {
@@ -274,7 +285,7 @@ pub fn apply_lambda_with_rows_output<'a>(
                     None => &null_row,
                 }
             }
-            Err(e) => panic!("python function failed {}", e),
+            Err(e) => panic!("python function failed {e}"),
         }
     });
 
