@@ -307,7 +307,7 @@ impl PySeries {
                     }
                     previous = o;
                 }
-                let mut out = ListChunked::from_chunks(name, vec![arr]);
+                let mut out = unsafe { ListChunked::from_chunks(name, vec![arr]) };
                 if fast_explode {
                     out.set_fast_explode()
                 }
@@ -1136,6 +1136,7 @@ impl PySeries {
         let options = SortOptions {
             descending: reverse,
             nulls_last: reverse,
+            multithreaded: true,
         };
         self.series.is_sorted(options)
     }
