@@ -139,9 +139,9 @@ def test_null_handling_correlation() -> None:
     df1 = pl.DataFrame({"a": [None, 1, 2], "b": [None, 2, 1]})
     df2 = pl.DataFrame({"a": [np.nan, 1, 2], "b": [np.nan, 2, 1]})
 
-    assert np.isclose(df1.select(pl.spearman_rank_corr("a", "b"))[0, 0], -1.0)
+    assert np.isclose(df1.select(pl.spearman_rank_corr("a", "b")).item(), -1.0)
     assert (
-        str(df2.select(pl.spearman_rank_corr("a", "b", propagate_nans=True))[0, 0])
+        str(df2.select(pl.spearman_rank_corr("a", "b", propagate_nans=True)).item())
         == "nan"
     )
 
